/*
 * gehl.h
 * Graphics Easy Hangling Library
 */
#ifndef _gehl_h
#define _gehl_h

#ifdef __cplusplus
extern "C"{
#endif


/* definition of file format */
#define GEHL_FILEFORMAT_FAILED  -2 /* failed to check */
#define GEHL_FILEFORMAT_UNKNOWN -1 /* unknown format */
#define GEHL_FILEFORMAT_WINDIB   1 /* Windows DIB (.bmp) */
#define GEHL_FILEFORMAT_JPEG     2 /* JPEG (.jpg, .jpeg) */
#define GEHL_FILEFORMAT_TIFF     3 /* TIFF (.tif, .tiff) */
#define GEHL_FILEFORMAT_PNG      4 /* PNG (.png) */


/* load image */
extern int
GEHL_Load(const char *fname, GEHL_IMG *img);

/* save image as Windows bitmap */
extern int
GEHL_Save(const char *fname, const GEHL_IMG *img, int format);

/* free image */
extern int
GEHL_Free(GEHL_IMG *img);

/* turnover VERTICAL/HORIZONTAL */
extern int
GEHL_Turnover(GEHL_IMG *img, int direction);

/* rotate 1-359 degree */
extern int
GEHL_Rotate(GEHL_IMG *img, int degree);

/* trim (x, y)-(x+w, y+h) */
extern int
GEHL_Trim(GEHL_IMG *img,
          int x, int y,
          int w, int h);

/* color to mono (24bit only) */
extern int
GEHL_C2Mono(GEHL_IMG *img, int border);

/* color to grayscale (24bit only) */
extern int
GEHL_C2Gray(GEHL_IMG *img);

/* copy */
extern int
GEHL_Copy(GEHL_IMG *dest, const GEHL_IMG *src);


#ifdef __cplusplus
}
#endif

#endif /* _gehl_h */
